@@ -20,41 +20,41 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "drvi.h"	/* For LINUXFLOPPY and WIN32FLOPPY */
+#include "drvi.h"   /* For LINUXFLOPPY and WIN32FLOPPY */
 #include "compi.h"
 #include "comp.h"
 /* LibDsk generalised compression support */
-#ifdef LINUXFLOPPY
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
 
 #define TMPDIR "/tmp"
 
-static COMPRESS_CLASS *classes[] = 
+static COMPRESS_CLASS *classes[] =
 {
 
 #include "compress.inc"
 
-	NULL
+    NULL
 };
 
 
 static dsk_err_t comp_construct(COMPRESS_DATA *cd, const char *filename)
 {
-	cd->cd_cfilename = dsk_malloc(1 + strlen(filename));	
-	if (!cd->cd_cfilename) return DSK_ERR_NOMEM;
-	strcpy(cd->cd_cfilename, filename);
-	cd->cd_ufilename = NULL;
-	cd->cd_readonly = 0;
-	return DSK_ERR_OK;
+    cd->cd_cfilename = dsk_malloc(1 + strlen(filename));    
+    if (!cd->cd_cfilename) return DSK_ERR_NOMEM;
+    strcpy(cd->cd_cfilename, filename);
+    cd->cd_ufilename = NULL;
+    cd->cd_readonly = 0;
+    return DSK_ERR_OK;
 }
 
 void comp_free(COMPRESS_DATA *cd)
 {
-	if (!cd) return;
-	if (cd->cd_cfilename) free(cd->cd_cfilename);
-	if (cd->cd_ufilename) free(cd->cd_ufilename);
-	free(cd);
+    if (!cd) return;
+    if (cd->cd_cfilename) free(cd->cd_cfilename);
+    if (cd->cd_ufilename) free(cd->cd_ufilename);
+    free(cd);
 }
 
 
@@ -71,19 +71,19 @@ static dsk_err_t comp_iopen(COMPRESS_DATA **cd, const char *filename, int nc)
 	memset((*cd), 0, cc->cc_selfsize);
         err = comp_construct(*cd, filename);
 	(*cd)->cd_class = cc;
-	if (err == DSK_ERR_OK) 
-	{
-		char *s = dsk_malloc(strlen(cc->cc_description) + 50);
-		if (s) 
-		{
-			sprintf(s, "Checking compression: %s...", cc->cc_description);
-			dsk_report(s);
-			dsk_free(s);
-		}
-		else	dsk_report("Checking compression...");
-		err = (cc->cc_open)(*cd);
-		dsk_report_end();
-	}
+    if (err == DSK_ERR_OK) 
+    {
+        char *s = dsk_malloc(strlen(cc->cc_description) + 50);
+        if (s) 
+        {
+            sprintf(s, "Checking compression: %s...", cc->cc_description);
+            dsk_report(s);
+            dsk_free(s);
+        }
+        else    dsk_report("Checking compression...");
+        err = (cc->cc_open)(*cd);
+        dsk_report_end();
+    }
         if (err == DSK_ERR_OK) return err;
         comp_free (*cd);
         *cd = NULL;
@@ -103,13 +103,13 @@ static dsk_err_t comp_icreat(COMPRESS_DATA **cd, const char *filename, int nc)
         if (!*cd) return DSK_ERR_NOMEM;
 	memset((*cd), 0, cc->cc_selfsize);
         err = comp_construct(*cd, filename);
-	(*cd)->cd_class = cc;
-	if (err == DSK_ERR_OK) err = (cc->cc_creat)(*cd);
+    (*cd)->cd_class = cc;
+    if (err == DSK_ERR_OK) err = (cc->cc_creat)(*cd);
 /* Stake out our claim to the temporary file. */
         if (err == DSK_ERR_OK) err = comp_mktemp(*cd, &fp);
-	if (fp) fclose(fp);	
-	if (err == DSK_ERR_OK) return err;
-	
+    if (fp) fclose(fp);
+    if (err == DSK_ERR_OK) return err;
+    
         comp_free (*cd);
         *cd = NULL;
         return err;
@@ -119,93 +119,100 @@ static dsk_err_t comp_icreat(COMPRESS_DATA **cd, const char *filename, int nc)
 
 dsk_err_t comp_open(COMPRESS_DATA **cd, const char *filename, const char *type)
 {
-	int nc;
-	dsk_err_t e;
+    int nc;
+    dsk_err_t e;
+    struct stat st;
+
+    if (!cd || !filename) return DSK_ERR_BADPTR;
+    *cd = NULL;
+
+/* Do not attempt to decompress directories */  
+#ifdef __PACIFIC__
+    /* Pacific doesn't declare the first parameter to stat() as const */
+    if (stat((char *)filename, &st)) return DSK_ERR_NOTME;
+#else
+    if (stat(filename, &st)) return DSK_ERR_NOTME;
+#endif
+    if (S_ISDIR(st.st_mode)) return DSK_ERR_NOTME;
 
-	if (!cd || !filename) return DSK_ERR_BADPTR;
-	*cd = NULL;
 #ifdef LINUXFLOPPY 
 /* Do not try to decompress a floppy drive. The reason for this is: If the 
  * floppy has a format Linux can't detect, repeatedly failing to open /dev/fd0
  * as we check for each compression type causes long delays */
- 	{
-	        struct stat st;
-       		if (stat(filename, &st)) return DSK_ERR_NOTME;
-	        if (S_ISBLK(st.st_mode) && (major(st.st_rdev) == 2)) return DSK_ERR_NOTME;
-	}
+        if (S_ISBLK(st.st_mode) && (major(st.st_rdev) == 2)) return DSK_ERR_NOTME;
 #endif
 /* Same shortcut for Windows / DOS */
 #ifdef ANYFLOPPY
-        if (strlen(filename) == 2 || filename[1] == ':') return DSK_ERR_NOTME;
+        if (strlen(filename) == 2 && filename[1] == ':') return DSK_ERR_NOTME;
 #endif
-	
-	if (type)
-	{
-		for (nc = 0; classes[nc]; nc++)
-		{
-			if (!strcmp(type, classes[nc]->cc_name))
-				return comp_iopen(cd, filename, nc);
-		}
-		return DSK_ERR_NODRVR;
-	}
-	for (nc = 0; classes[nc]; nc++)
-	{
-		e = comp_iopen(cd, filename, nc);
-		if (e != DSK_ERR_NOTME) return e;
-	}	
-	return DSK_ERR_NOTME;
+    
+    if (type)
+    {
+        for (nc = 0; classes[nc]; nc++)
+        {
+            if (!strcmp(type, classes[nc]->cc_name))
+                return comp_iopen(cd, filename, nc);
+        }
+        return DSK_ERR_NODRVR;
+    }
+    for (nc = 0; classes[nc]; nc++)
+    {
+        e = comp_iopen(cd, filename, nc);
+        if (e != DSK_ERR_NOTME) return e;
+    }   
+    return DSK_ERR_NOTME;
 }
 
 dsk_err_t comp_creat(COMPRESS_DATA **cd, const char *filename, const char *type)
 {
-	int nc;
+    int nc;
 
-	if (!type)	/* Uncompressed */
-	{
-		*cd = NULL;
-		return DSK_ERR_OK;
-	}
+    if (!type)  /* Uncompressed */
+    {
+        *cd = NULL;
+        return DSK_ERR_OK;
+    }
 
-	if (!cd || !filename) return DSK_ERR_BADPTR;
-	
-	for (nc = 0; classes[nc]; nc++)
-	{
-		if (!strcmp(type, classes[nc]->cc_name))
-			return comp_icreat(cd, filename, nc);
-	}
-	return DSK_ERR_NODRVR;
+    if (!cd || !filename) return DSK_ERR_BADPTR;
+    
+    for (nc = 0; classes[nc]; nc++)
+    {
+        if (!strcmp(type, classes[nc]->cc_name))
+            return comp_icreat(cd, filename, nc);
+    }
+    return DSK_ERR_NODRVR;
 }
 
 
 
 dsk_err_t comp_commit(COMPRESS_DATA **self)
 {
-	dsk_err_t e;
+    dsk_err_t e;
 
-	if (!self || (!(*self)) || (!(*self)->cd_class))    return DSK_ERR_BADPTR;
+    if (!self || (!(*self)) || (!(*self)->cd_class))    return DSK_ERR_BADPTR;
 
-	dsk_report("Compressing...");
-	e = ((*self)->cd_class->cc_commit)(*self);
-	dsk_report_end();
+    dsk_report("Compressing...");
+    e = ((*self)->cd_class->cc_commit)(*self);
+    dsk_report_end();
 
-	if ((*self)->cd_ufilename) remove((*self)->cd_ufilename);
-	comp_free (*self);
-	*self = NULL;
-	return e;
+    if ((*self)->cd_ufilename) remove((*self)->cd_ufilename);
+    comp_free (*self);
+    *self = NULL;
+    return e;
 }
 
 dsk_err_t comp_abort(COMPRESS_DATA **self)
 {
-	dsk_err_t e;
+    dsk_err_t e;
 
-	if (!self || (!(*self)) || (!(*self)->cd_class))    return DSK_ERR_BADPTR;
+    if (!self || (!(*self)) || (!(*self)->cd_class))    return DSK_ERR_BADPTR;
 
-	e = ((*self)->cd_class->cc_abort)(*self);
+    e = ((*self)->cd_class->cc_abort)(*self);
 
-	if ((*self)->cd_ufilename) remove((*self)->cd_ufilename);
-	comp_free (*self);
-	*self = NULL;
-	return e;
+    if ((*self)->cd_ufilename) remove((*self)->cd_ufilename);
+    comp_free (*self);
+    *self = NULL;
+    return e;
 }
 
 
@@ -213,36 +220,36 @@ dsk_err_t comp_abort(COMPRESS_DATA **self)
  * Else sets (*drvname) to null. */
 dsk_err_t comp_type_enum(int index, char **compname)
 {
-	int nc;
+    int nc;
 
-	if (!compname) return DSK_ERR_BADPTR;
+    if (!compname) return DSK_ERR_BADPTR;
 
-	for (nc = 0; classes[nc]; nc++)
-	{
-		if (index == nc)
-		{
-			*compname = classes[nc]->cc_name;
-			return DSK_ERR_OK;
-		}
-	}
-	*compname = NULL;
-	return DSK_ERR_NODRVR;
+    for (nc = 0; classes[nc]; nc++)
+    {
+        if (index == nc)
+        {
+            *compname = classes[nc]->cc_name;
+            return DSK_ERR_OK;
+        }
+    }
+    *compname = NULL;
+    return DSK_ERR_NODRVR;
 }
 
 
 const char *comp_name(COMPRESS_DATA *self)
 {
-	if (!self || !self->cd_class || !self->cd_class->cc_name)
-		return "(null)";
-	return self->cd_class->cc_name;
+    if (!self || !self->cd_class || !self->cd_class->cc_name)
+        return "(null)";
+    return self->cd_class->cc_name;
 }
 
 
 const char *comp_desc(COMPRESS_DATA *self)
 {
-	if (!self || !self->cd_class || !self->cd_class->cc_description)
-		return "(null)";
-	return self->cd_class->cc_description;
+    if (!self || !self->cd_class || !self->cd_class->cc_description)
+        return "(null)";
+    return self->cd_class->cc_description;
 }
 
 
@@ -256,61 +263,61 @@ dsk_err_t comp_fopen(COMPRESS_DATA *self, FILE **fp)
                 (*fp) = fopen(self->cd_cfilename, "rb");
         }
         if (!(*fp)) return DSK_ERR_SYSERR;
-	return DSK_ERR_OK;
+    return DSK_ERR_OK;
 }
     
     
 dsk_err_t comp_mktemp(COMPRESS_DATA *self, FILE **fp)
 {
-	char *tdir;
-	int fd;
-	char tmpdir[PATH_MAX];
+    char *tdir;
+    int fd;
+    char tmpdir[PATH_MAX];
 
-	self->cd_ufilename = dsk_malloc(PATH_MAX);
+    self->cd_ufilename = dsk_malloc(PATH_MAX);
 
 /* Win32: Create temp file using GetTempFileName() */
 #ifdef HAVE_GETTEMPFILENAME
         GetTempPath(PATH_MAX, tmpdir);
         if (!GetTempFileName(tmpdir, "dsk", 0, self->cd_ufilename))
-	{
-		dsk_free(self->cd_ufilename);
-		self->cd_ufilename = NULL;
-		return DSK_ERR_SYSERR;
-	}
-       	*fp = fopen(self->cd_ufilename, "wb");
-		(void)fd;
-		(void)tdir;
+    {
+        dsk_free(self->cd_ufilename);
+        self->cd_ufilename = NULL;
+        return DSK_ERR_SYSERR;
+    }
+        *fp = fopen(self->cd_ufilename, "wb");
+        (void)fd;
+        (void)tdir;
 #else /* def HAVE_GETTEMPFILENAME */
 
 /* Modern Unixes: Use mkstemp() */
 #ifdef HAVE_MKSTEMP
-	tdir = getenv("TMPDIR");
-	if (tdir) sprintf(tmpdir, "%s/libdskdXXXXXXXX", tdir);
-	else      sprintf(tmpdir, TMPDIR "/libdskXXXXXXXX");
+    tdir = getenv("TMPDIR");
+    if (tdir) sprintf(tmpdir, "%s/libdskdXXXXXXXX", tdir);
+    else      sprintf(tmpdir, TMPDIR "/libdskXXXXXXXX");
 
-	fd = mkstemp(tmpdir);
-	*fp = NULL;
-	if (fd != -1) fp[0] = fdopen(fd, "wb");
-	strcpy(self->cd_ufilename, tmpdir);
-#else	/* def HAVE_MKSTEMP */
+    fd = mkstemp(tmpdir);
+    *fp = NULL;
+    if (fd != -1) fp[0] = fdopen(fd, "wb");
+    strcpy(self->cd_ufilename, tmpdir);
+#else   /* def HAVE_MKSTEMP */
 /* Old Unixes, old xenix clones such as DOS */ 
-	tdir = getenv("TMP");
-	if (!tdir) tdir = getenv("TEMP");
-	if (tdir) sprintf(tmpdir, "%s/LDXXXXXX", tdir);
-	else      sprintf(tmpdir, "./LDXXXXXX");
+    tdir = getenv("TMP");
+    if (!tdir) tdir = getenv("TEMP");
+    if (tdir) sprintf(tmpdir, "%s/LDXXXXXX", tdir);
+    else      sprintf(tmpdir, "./LDXXXXXX");
 
-	strcpy(self->cd_ufilename, mktemp(tmpdir));
-	fp[0] = fopen(self->cd_ufilename, "wb");
-   	(void)fd;
+    strcpy(self->cd_ufilename, mktemp(tmpdir));
+    fp[0] = fopen(self->cd_ufilename, "wb");
+    (void)fd;
 #endif /* HAVE_MKSTEMP */                  
 #endif /* HAVE_GETTEMPFILENAME */
-	if (!*fp)	
-	{
-		dsk_free(self->cd_ufilename);
-		self->cd_ufilename = NULL;
-		return DSK_ERR_SYSERR;
-	}
-	return DSK_ERR_OK;
+    if (!*fp)   
+    {
+        dsk_free(self->cd_ufilename);
+        self->cd_ufilename = NULL;
+        return DSK_ERR_SYSERR;
+    }
+    return DSK_ERR_OK;
 }
 
 
