@@ -33,6 +33,10 @@
 # include <limits.h>
 #endif
 
+#ifdef HAVE_ASSERT_H
+# include <assert.h>
+#endif
+
 #ifndef DISABLE_FLOPPY
 # ifdef HAVE_LINUX_FD_H
 #  include "linux/fd.h"
@@ -48,6 +52,7 @@
 
 #ifdef HAVE_WINIOCTL_H
 #  define WIN32FLOPPY 
+#  define NTWDMFLOPPY 
 #  include <winioctl.h>
 # endif
 #endif
@@ -66,3 +71,42 @@
 # define ANYFLOPPY
 #endif
 
+#if defined(HAVE_UNISTD_H)
+#define HAVE_RCPMFS 1
+#elif defined(_WIN32)
+#define HAVE_RCPMFS 1
+#elif defined(HAVE_DIR_H)
+#define HAVE_RCPMFS 1
+#endif
+
+
+/* Initialise custom formats */
+dsk_err_t dg_custom_init(void);
+const char *dg_homedir(void);
+const char *dg_sharedir(void);
+dsk_err_t dg_parseline(char *linebuf, DSK_GEOMETRY *dg, char *description);
+dsk_err_t dg_parse(FILE *fp, DSK_GEOMETRY *dg, char *description);
+dsk_err_t dg_store(FILE *fp, DSK_GEOMETRY *dg, char *description);
+/* The default geometry probe; driver geometry probes can call it */
+dsk_err_t dsk_defgetgeom(DSK_DRIVER *self, DSK_GEOMETRY *geom);
+/* The default system for storing optional integer properties */
+dsk_err_t dsk_isetoption(DSK_DRIVER *self, const char *name, int value, 
+		int add_if_not_present);
+
+
+/* "Extended surface" discs have sector IDs that don't match the sectors' 
+ * location on disc. This means, anywhere we reflect dsk_read() to dsk_xread(),
+ * we have to translate the physical location to the sector ID field rather 
+ * than assuming a 1:1 mapping. */
+
+dsk_phead_t dg_x_head  (const DSK_GEOMETRY *dg, dsk_phead_t h);
+dsk_psect_t dg_x_sector(const DSK_GEOMETRY *dg, dsk_phead_t h, dsk_psect_t s);
+
+
+#ifdef AUTOSHARE
+# define Q2(x) Q1(x)
+# define Q1(x) #x
+# define SHAREDIR Q2(AUTOSHARE)
+#else
+# define SHAREDIR NULL
+#endif
