@@ -155,7 +155,9 @@ static dsk_err_t check_geom(LINUX_DSK_DRIVER *self, const DSK_GEOMETRY *dg)
 	str.fmt_gap = dg->dg_fmtgap;
 
 	if (ioctl(self->lx_fd, FDSETPRM, &str)) return DSK_ERR_SYSERR;
-
+/* 1.1.10: Make the next command do a seek, in case the FDSETPRM
+ * has moved the head. */
+	self->lx_cylinder = ~0;
 	memcpy(&self->lx_geom, dg, sizeof(*dg));
 	return DSK_ERR_OK;
 }
@@ -191,6 +193,9 @@ dsk_err_t linux_open(DSK_DRIVER *self, const char *filename)
 		lxself->lx_fd = open(filename, O_NONBLOCK | O_RDONLY);
 	}
 	if (lxself->lx_fd < 0) return DSK_ERR_SYSERR;
+/* [v1.3.1] This appears to be necessary, otherwise an aborted dskscan will
+ * not necessarily seek back to cylinder 0 */
+	ioctl(lxself->lx_fd, FDCLRPRM);
 
 	return DSK_ERR_OK;
 }
@@ -230,8 +235,9 @@ dsk_err_t linux_read(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
                              void *buf, dsk_pcyl_t cylinder,
                               dsk_phead_t head, dsk_psect_t sector)
 {
-	return linux_xread(self, geom, buf, cylinder, head, 
-			   cylinder, head, sector, geom->dg_secsize, 0);
+	return linux_xread(self, geom, buf, cylinder, head, cylinder, 
+			dg_x_head(geom, head), 
+			dg_x_sector(geom, head, sector), geom->dg_secsize, 0);
 }
 
 dsk_err_t linux_xread(DSK_DRIVER *self, const DSK_GEOMETRY *geom, void *buf,
@@ -272,13 +278,14 @@ dsk_err_t linux_xread(DSK_DRIVER *self, const DSK_GEOMETRY *geom, void *buf,
 	raw_cmd.cmd[raw_cmd.cmd_count++] = cyl_expected;
 	raw_cmd.cmd[raw_cmd.cmd_count++] = head_expected;
 	raw_cmd.cmd[raw_cmd.cmd_count++] = sector;
-	raw_cmd.cmd[raw_cmd.cmd_count++] = dsk_get_psh(geom->dg_secsize);
+	raw_cmd.cmd[raw_cmd.cmd_count++] = dsk_get_psh(sector_size);
 	raw_cmd.cmd[raw_cmd.cmd_count++] = sector;
 	raw_cmd.cmd[raw_cmd.cmd_count++] = geom->dg_rwgap;
 	raw_cmd.cmd[raw_cmd.cmd_count++] = (sector_size < 255) ? sector_size : 0xFF; 
 
 	if (ioctl(lxself->lx_fd, FDRAWCMD, &raw_cmd) < 0) return DSK_ERR_SYSERR;
 
+	memcpy(lxself->lx_status, raw_cmd.reply, 4);
 	if (raw_cmd.reply[0] & 0x40) return xlt_error(raw_cmd.reply);
 
 	if (deleted) deleted[0] = (raw_cmd.reply[2] & 0x40) ? 1 : 0;
@@ -291,8 +298,10 @@ dsk_err_t linux_write(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
                              const void *buf, dsk_pcyl_t cylinder,
                               dsk_phead_t head, dsk_psect_t sector)
 {
-	return linux_xwrite(self, geom, buf, cylinder, head, cylinder, head,
-				sector, geom->dg_secsize, 0);
+	return linux_xwrite(self, geom, buf, cylinder, head, cylinder, 
+			dg_x_head(geom, head),
+			dg_x_sector(geom, head, sector), 
+			geom->dg_secsize, 0);
 }
 
 
@@ -333,13 +342,14 @@ dsk_err_t linux_xwrite(DSK_DRIVER *self, const DSK_GEOMETRY *geom, const void *b
 	raw_cmd.cmd[raw_cmd.cmd_count++] = cyl_expected;
 	raw_cmd.cmd[raw_cmd.cmd_count++] = head_expected;
 	raw_cmd.cmd[raw_cmd.cmd_count++] = sector;
-	raw_cmd.cmd[raw_cmd.cmd_count++] = dsk_get_psh(geom->dg_secsize);
+	raw_cmd.cmd[raw_cmd.cmd_count++] = dsk_get_psh(sector_size);
 	raw_cmd.cmd[raw_cmd.cmd_count++] = sector;
 	raw_cmd.cmd[raw_cmd.cmd_count++] = geom->dg_rwgap;
 	raw_cmd.cmd[raw_cmd.cmd_count++] = (sector_size < 255) ? sector_size : 0xFF; 
 
 	if (ioctl(lxself->lx_fd, FDRAWCMD, &raw_cmd) < 0) return DSK_ERR_SYSERR;
 
+	memcpy(lxself->lx_status, raw_cmd.reply, 4);
 	if (raw_cmd.reply[0] & 0x40) return xlt_error(raw_cmd.reply);
 	lxself->lx_cylinder = cylinder;
 	return DSK_ERR_OK;
@@ -397,6 +407,7 @@ dsk_err_t linux_format(DSK_DRIVER *self, DSK_GEOMETRY *geom,
 
 	if (ioctl(lxself->lx_fd, FDRAWCMD, &raw_cmd) < 0) return DSK_ERR_SYSERR;
 
+	memcpy(lxself->lx_status, raw_cmd.reply, 4);
 	if (raw_cmd.reply[0] & 0x40) return xlt_error(raw_cmd.reply);
 	lxself->lx_cylinder = cylinder;
 	return DSK_ERR_OK;
@@ -441,6 +452,7 @@ dsk_err_t linux_secid(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
 
 	if (ioctl(lxself->lx_fd, FDRAWCMD, &raw_cmd) < 0) return DSK_ERR_SYSERR;
 
+	memcpy(lxself->lx_status, raw_cmd.reply, 4);
 	if (raw_cmd.reply[0] & 0x40) return xlt_error(raw_cmd.reply);
 	result->fmt_cylinder = raw_cmd.reply[3];	
 	result->fmt_head     = raw_cmd.reply[4];	
@@ -475,6 +487,7 @@ dsk_err_t linux_xseek(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
 
 	if (ioctl(lxself->lx_fd, FDRAWCMD, &raw_cmd) < 0) return DSK_ERR_SYSERR;
 
+	memcpy(lxself->lx_status, raw_cmd.reply, 4);
 	if (raw_cmd.reply[0] & 0x40) return xlt_error(raw_cmd.reply);
 	lxself->lx_cylinder = cylinder;
 	return DSK_ERR_OK;
@@ -553,6 +566,7 @@ dsk_err_t linux_xtread(DSK_DRIVER *self, const DSK_GEOMETRY *geom, void *buf,
 
 	if (ioctl(lxself->lx_fd, FDRAWCMD, &raw_cmd) < 0) return DSK_ERR_SYSERR;
 
+	memcpy(lxself->lx_status, raw_cmd.reply, 4);
 	if (raw_cmd.reply[0] & 0x40) return xlt_error(raw_cmd.reply);
 	lxself->lx_cylinder = cylinder;
 	return DSK_ERR_OK;
@@ -568,6 +582,10 @@ dsk_err_t linux_option_enum(DSK_DRIVER *self, int idx, char **optname)
 	{
 		case 0: if (optname) *optname = "HEAD"; return DSK_ERR_OK;
 		case 1: if (optname) *optname = "DOUBLESTEP"; return DSK_ERR_OK;
+		case 2: if (optname) *optname = "ST0"; return DSK_ERR_OK;
+		case 3: if (optname) *optname = "ST1"; return DSK_ERR_OK;
+		case 4: if (optname) *optname = "ST2"; return DSK_ERR_OK;
+		case 5: if (optname) *optname = "ST3"; return DSK_ERR_OK;
 	}
 	return DSK_ERR_BADOPT;	
 
@@ -595,6 +613,11 @@ dsk_err_t linux_option_set(DSK_DRIVER *self, const char *optname, int value)
 			return DSK_ERR_OK;
 		default: return DSK_ERR_BADVAL;
 	}
+/* ST0 - ST3 are read-only options. No values can be set for them. */
+	if ((!strcmp(optname, "ST0")) ||
+	    (!strcmp(optname, "ST1")) ||
+	    (!strcmp(optname, "ST2")) ||
+	    (!strcmp(optname, "ST3"))) return DSK_ERR_BADVAL;
 	return DSK_ERR_BADOPT;
 }
 	
@@ -616,6 +639,26 @@ dsk_err_t linux_option_get(DSK_DRIVER *self, const char *optname, int *value)
 		if (value) *value = lxself->lx_doublestep;
 		return DSK_ERR_OK;
 	}
+	if (!strcmp(optname, "ST0"))
+	{
+		if (value) *value = lxself->lx_status[0];
+		return DSK_ERR_OK;
+	}
+	if (!strcmp(optname, "ST1"))
+	{
+		if (value) *value = lxself->lx_status[1];
+		return DSK_ERR_OK;
+	}
+	if (!strcmp(optname, "ST2"))
+	{
+		if (value) *value = lxself->lx_status[2];
+		return DSK_ERR_OK;
+	}
+	if (!strcmp(optname, "ST3"))
+	{
+		if (value) *value = lxself->lx_status[3];
+		return DSK_ERR_OK;
+	}
 	return DSK_ERR_BADOPT;
 }
 	
