@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2005  John Elliott <jce@seasip.demon.co.uk>            *
+ *    Copyright (C) 2005,2008,2010-11  John Elliott <jce@seasip.demon.co.uk> *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -72,6 +72,11 @@
 #define CONFIGFILE ".libdsk.ini"
 #define BOOTFILE   ".libdsk.boot"
 
+static const int DIR_EX = 12;	/* Extent counter (low bits) */
+static const int DIR_S1 = 13;	/* Last record byte count */
+static const int DIR_S2 = 14;	/* Extent counter (high bits) */
+static const int DIR_RC = 15;	/* Record counter */
+
 /* Filename map entry holds real name. We allow 13 bytes for filename.type, 
    plus 4 bytes for the user number. User numbers are done by prepending 
    the number and then two dots; so 10:filename.typ would become 
@@ -118,6 +123,14 @@ DRV_CLASS dc_rcpmfs =
 	rcpmfs_secid,	/* sector ID */
 	rcpmfs_xseek,   /* seek to track */
 	rcpmfs_status,  /* drive status */
+	NULL,		/* xread */
+	NULL,		/* xwrite */
+	NULL,		/* tread */
+	NULL,		/* xtread */
+	rcpmfs_option_enum,	/* List options */
+	rcpmfs_option_set,	/* Set option */
+	rcpmfs_option_get,	/* Get option */
+
 };
 
 
@@ -165,7 +178,7 @@ static unsigned extent_bytes(RCPMFS_DSK_DRIVER *self, unsigned char *dirent)
 {
 	unsigned exm = rcpmfs_get_exm(self);
 
-	return (dirent[15] + ((dirent[12] & exm) * 128)) * 128;
+	return (dirent[DIR_RC] + ((dirent[DIR_EX] & exm) * 128)) * 128;
 }
 
 
@@ -232,6 +245,14 @@ static dsk_err_t rcpmfs_adjust_size(RCPMFS_DSK_DRIVER *self,
 	struct stat st;
 	long newsize;
 
+	/* ISX, annoyingly, gives the last record byte count the opposite
+	 * meaning. That is, it is the number of _unused_ bytes in the last
+	 * record. */
+	if (self->rc_fsversion == FSVERSION_ISX)
+	{
+		lrbc = (128 - lrbc) & 0x7F;
+	}
+
 /* Adjust the size of a file (to within 128 bytes). 
  * Rather than do this by working out how big CP/M thinks the file should
  * be, we stat() it, get the real size, and tweak that. */
@@ -362,7 +383,11 @@ static dsk_err_t rcpmfs_option(RCPMFS_DSK_DRIVER *self, char *variable,
 	if (!strcmp(variable, "systracks"))	/* We are allowed systracks=0*/ 
 		   self->rc_systracks = atoi(value);	
 	if (!strcmp(variable, "version") && atoi(value)) 
-		   self->rc_fsversion = atoi(value);	
+		   self->rc_fsversion = atoi(value);
+	else if (!strcmp(variable, "version") && !strcmp(value, "isx"))
+		   self->rc_fsversion = FSVERSION_ISX;
+	else if (!strcmp(variable, "version") && !strcmp(value, "ISX"))
+		   self->rc_fsversion = FSVERSION_ISX;
 
 	if (!strcmp(variable, "format"))
 	{
@@ -411,8 +436,9 @@ static dsk_err_t rcpmfs_dump_options(RCPMFS_DSK_DRIVER *self, FILE *fp)
 	                        self->rc_totalblocks);
 	fprintf(fp, "SysTracks=%u    ; Number of system tracks\n", 
 	                    self->rc_systracks);
-	fprintf(fp, "Version=%u      ; Filesystem version (CP/M 2 or 3)\n", 
-	                    self->rc_fsversion);
+	if (self->rc_fsversion == FSVERSION_ISX)
+		fprintf(fp, "Version=ISX    ; Filesystem version (CP/M 2 or 3, or ISX)\n");
+	else	fprintf(fp, "Version=%u      ; Filesystem version (CP/M 2 or 3, or ISX)\n", self->rc_fsversion);
 
 	do
 	{
@@ -488,6 +514,7 @@ static dsk_err_t rcpmfs_parse(RCPMFS_DSK_DRIVER *self, FILE *fp)
 /******************** CP/M DIRECTORY ACCESS ************************/
 /* Look up a block in the CP/M directory and say what file owns it */
 
+#include <assert.h>
 
 static dsk_err_t rcpmfs_writebuffer(RCPMFS_DSK_DRIVER *self,
 		const void *data, dsk_lsect_t lsect)
@@ -500,6 +527,13 @@ static dsk_err_t rcpmfs_writebuffer(RCPMFS_DSK_DRIVER *self,
 	{
 		if (rcb->rcb_lsect == lsect)
 		{
+/* This is a "can't happen" error - trying to write a sector size other than
+ * the one we're using */
+			assert(rcb->rcb_size == self->rc_geom.dg_secsize);
+			if (rcb->rcb_size != self->rc_geom.dg_secsize)
+			{
+				return DSK_ERR_ECHECK;
+			}
 			memcpy(rcb->rcb_data, data, self->rc_geom.dg_secsize);
 			return DSK_ERR_OK;
 		}
@@ -510,6 +544,7 @@ static dsk_err_t rcpmfs_writebuffer(RCPMFS_DSK_DRIVER *self,
 	if (!rcb) return DSK_ERR_NOMEM;
 	memcpy(rcb->rcb_data, data, self->rc_geom.dg_secsize);
 	rcb->rcb_next = NULL;
+	rcb->rcb_size = self->rc_geom.dg_secsize;
 	rcb->rcb_lsect = lsect;
 	RTRACE(("rcpmfs_writebuffer: Wrote %02x %02x %02x\n",
 		rcb->rcb_data[0], rcb->rcb_data[1], rcb->rcb_data[2]));
@@ -665,7 +700,13 @@ static int rcpmfs_83name(RCPMFS_DSK_DRIVER *self,
 		RTRACE(("Not a real file file\n"));
 		return 0;   /* Not a real file */
 	}
-	dirent[13] = (st->st_size & 0x7F);
+
+/* ISX gives the opposite meaning to the S1 byte */
+
+	if (self->rc_fsversion == FSVERSION_ISX)
+		dirent[DIR_S1] = (128 - (st->st_size & 0x7F)) & 0x7F;
+	else	dirent[DIR_S1] = (st->st_size & 0x7F);
+
 /* Set CP/M file attributes */
 #if defined(HAVE_DIR_H)
 	RTRACE(("Attributes from DOS attributes 0x%x\n", attributes)); 
@@ -925,11 +966,11 @@ static dsk_err_t rcpmfs_initdir(RCPMFS_DSK_DRIVER *self)
 	unsigned char label[32];
 	char *sep, *dst;
 	int n;
-	dsk_err_t err;
+	dsk_err_t err = DSK_ERR_OK;
 	struct stat st;
 
 	self->rc_dirent = 0;
-	if (self->rc_fsversion < 3)
+	if (self->rc_fsversion < FSVERSION_CPM3)
 	{
 		memset(label, 0x0E5, 32);
 		/* 1.2.1: 'entry' is unsigned, so ">= 0" is always true */
@@ -946,7 +987,7 @@ static dsk_err_t rcpmfs_initdir(RCPMFS_DSK_DRIVER *self)
 	sep = strrchr(self->rc_dir, SEPARATOR);
 	if (sep) ++sep;
 	else	 sep = self->rc_dir;
-	dst = label + 1;
+	dst = (char *)(label + 1);
 	for (n = 0; n < 11; n++)
 	{
 		if (!sep[0]) break;
@@ -1023,6 +1064,11 @@ static dsk_err_t rcpmfs_readdir(RCPMFS_DSK_DRIVER *self)
 
 	if (!self) return DSK_ERR_BADPTR;
 
+	if (self->rc_namemap)
+	{
+		dsk_free(self->rc_namemap);
+		self->rc_namemap = 0;
+	}
 	self->rc_namemap = dsk_malloc( NAMEMAP_ENTRYSIZE * 
 				rcpmfs_max_dirent(self));
 	if (!self->rc_namemap) return DSK_ERR_NOMEM;
@@ -1117,11 +1163,13 @@ static dsk_err_t rcpmfs_readdir(RCPMFS_DSK_DRIVER *self)
 /* Generate sizes for this extent */
 				extsize = rcpmfs_extent_size(self);
 				if (extsize > filesize) extsize = filesize;
-				cpm_dirent[12]  = (extent * (exm+1)) & 0x1F;
-				cpm_dirent[12] |= ((extsize + 127) / 16384) & exm;
-				cpm_dirent[13]  = (unsigned char)(filesize & 0x7F);
-				cpm_dirent[14]  = (extent * (exm+1)) / 32;
-				cpm_dirent[15]  = (unsigned char)((extsize + 127) / 128);
+				cpm_dirent[DIR_EX]  = (extent * (exm+1)) & 0x1F;
+				cpm_dirent[DIR_EX] |= ((extsize + 127) / 16384) & exm;
+				if (self->rc_fsversion == FSVERSION_ISX)
+					cpm_dirent[DIR_S1]  = (unsigned char)(128 - (filesize & 0x7F)) & 0x7F;
+				else	cpm_dirent[DIR_S1]  = (unsigned char)(filesize & 0x7F);
+				cpm_dirent[DIR_S2]  = (extent * (exm+1)) / 32;
+				cpm_dirent[DIR_RC]  = (unsigned char)((extsize + 127) / 128);
 				filesize -= extsize;
 				++extent;
 /* Add extent to the directory */
@@ -1233,7 +1281,8 @@ dsk_err_t rcpmfs_open(DSK_DRIVER *self, const char *passed)
 	rcself->rc_dirblocks = 2;
 	rcself->rc_totalblocks = 175;
 	rcself->rc_systracks = 1;
-	rcself->rc_fsversion = 3;
+	rcself->rc_fsversion = FSVERSION_CPM3;
+	rcself->rc_namemap = NULL;
 
 	/* Now we have to find out if there's a configuration file */
 	filename = rcpmfs_mkname(rcself, CONFIGFILE);
@@ -1300,7 +1349,8 @@ dsk_err_t rcpmfs_creat(DSK_DRIVER *self, const char *passed)
 	rcself->rc_dirblocks = 2;
 	rcself->rc_totalblocks = 175;
 	rcself->rc_systracks = 1;
-	rcself->rc_fsversion = 3;
+	rcself->rc_fsversion = FSVERSION_CPM3;
+	rcself->rc_namemap = NULL;
 /* 720k defaults 
 	err = dg_stdformat(&rcself->rc_geom, FMT_720K, NULL, NULL);
 	if (err) return err;
@@ -1308,7 +1358,7 @@ dsk_err_t rcpmfs_creat(DSK_DRIVER *self, const char *passed)
 	rcself->rc_dirblocks = 4;
 	rcself->rc_totalblocks = 357;
 	rcself->rc_systracks = 1;
-	rcself->rc_fsversion = 3;
+	rcself->rc_fsversion = FSVERSION_CPM3;
 */
 	/* Now we have to find out if there's a configuration file */
 	filename = rcpmfs_mkname(rcself, CONFIGFILE);
@@ -1334,6 +1384,20 @@ dsk_err_t rcpmfs_creat(DSK_DRIVER *self, const char *passed)
 
 
 
+static void rcpmfs_free_buffers(RCPMFS_DSK_DRIVER *self)
+{
+	RCPMFS_BUFFER *rcb, *rcb2;
+
+	/* Free buffers. The sector size may have changed */
+	rcb = self->rc_bufhead;
+	while (rcb)
+	{
+		rcb2 = rcb->rcb_next;
+		dsk_free(rcb);
+		rcb = rcb2;	
+	}
+	self->rc_bufhead = NULL;
+}
 
 
 dsk_err_t rcpmfs_close(DSK_DRIVER *self)
@@ -1394,7 +1458,7 @@ static dsk_err_t rcpmfs_psfind2(RCPMFS_DSK_DRIVER *self,
 		return DSK_ERR_OK;
 	}
 	/* Now to find the offset */
-	extent  = (dirent[12] & 0x1F) + (dirent[14] * 32);
+	extent  = (dirent[DIR_EX] & 0x1F) + (dirent[DIR_S2] * 32);
 	extent /= (exm + 1);
 
 	*offset  = diroffs;
@@ -1405,9 +1469,11 @@ static dsk_err_t rcpmfs_psfind2(RCPMFS_DSK_DRIVER *self,
 
 /* See how many bytes there are in the extent */
 	extent_len = extent_bytes(self, dirent);
-	if (dirent[13])
+	if (dirent[DIR_S1])
 	{
-		extent_len = (extent_len - 128) + dirent[13];
+		if (self->rc_fsversion == FSVERSION_ISX)
+			extent_len -= dirent[DIR_S1];
+		else	extent_len = (extent_len - 128) + dirent[DIR_S1];
 	}
 /* And reduce buffer size accordingly */
 	if (extent_len < (diroffs + blockoffs + bufsize[0]))
@@ -1418,7 +1484,7 @@ static dsk_err_t rcpmfs_psfind2(RCPMFS_DSK_DRIVER *self,
 	RTRACE(("Sector 0x%lx (block %lx) is in file %s 0x%02x 0x%02x 0x%02x "
 		" -> 0x%lx len 0x%lx\n", 
 			lsect, blockno, fnbuf,
-			dirent[12], dirent[13], dirent[14],
+			dirent[DIR_EX], dirent[DIR_S1], dirent[DIR_S2],
 			(unsigned long)(*offset), (unsigned long)(*bufsize)));
 
 	return DSK_ERR_OK;
@@ -1580,7 +1646,10 @@ static dsk_err_t rcpmfs_chgdir(RCPMFS_DSK_DRIVER *self,
 		unsigned entryno, unsigned char *old, unsigned char *new)
 {
 	dsk_err_t err;
-	char realname[14], newname[14];
+	// JSIE-Fix: buffer too short to hold cp/m file names including user 
+	// number and two more dots to separate user number from file name 
+	//  char realname[14], newname[14];
+	char realname[17], newname[17];
 	FILE *fp;
 	unsigned exm;
 	unsigned oldextent, newextent;
@@ -1595,8 +1664,8 @@ static dsk_err_t rcpmfs_chgdir(RCPMFS_DSK_DRIVER *self,
 
 
 	exm	  = rcpmfs_get_exm(self);
-	oldextent = ((old[14] * 32) + (old[12] & 31)) / (exm+1);
-	newextent = ((new[14] * 32) + (new[12] & 31)) / (exm+1);
+	oldextent = ((old[DIR_S2] * 32) + (old[DIR_EX] & 31)) / (exm+1);
+	newextent = ((new[DIR_S2] * 32) + (new[DIR_EX] & 31)) / (exm+1);
 
 	RTRACE(("oldextent=%d newextent=%d\n", oldextent, newextent));
 
@@ -1606,7 +1675,7 @@ static dsk_err_t rcpmfs_chgdir(RCPMFS_DSK_DRIVER *self,
 	}
 	if (new[0] == 0x20) /* Directory label being written */
 	{
-		self->rc_dirlabel = new[12];
+		self->rc_dirlabel = new[DIR_EX];
 	}
 /* Creation and deletion requests. We only handle these for extent 0 */
 	if (old[0] >= 0x10 && new[0] < 0x10 && (newextent == 0)) /* new file */
@@ -1718,13 +1787,13 @@ RTRACE(("Rename '%s' as '%s'", realname, newname));
 	{
 		rcpmfs_cpmname(new, realname);
 		RTRACE(("Reduce file size: %s oldlen=%ld newlen=%ld\n", rcpmfs_mkname(self,realname), oldlen, newlen));
-		err = rcpmfs_adjust_size(self, oldlen - newlen, new[13], rcpmfs_mkname(self,realname));
+		err = rcpmfs_adjust_size(self, oldlen - newlen, new[DIR_S1], rcpmfs_mkname(self,realname));
 	}
 /* File remains roughly, the same size, but Last Record Byte Count tweaked. */
 	else if (old[0x0d] != new[0x0d] && (newextent == 0))	
 	{
 		rcpmfs_cpmname(new, realname);
-		err = rcpmfs_adjust_size(self, 0, new[13], rcpmfs_mkname(self,realname));
+		err = rcpmfs_adjust_size(self, 0, new[DIR_S1], rcpmfs_mkname(self,realname));
 	}
 	return DSK_ERR_OK;
 }
@@ -1919,10 +1988,23 @@ dsk_err_t rcpmfs_format(DSK_DRIVER *self, DSK_GEOMETRY *geom,
 	    geom->dg_secsize   != rcself->rc_geom.dg_secsize   ||
 	    geom->dg_secbase   != rcself->rc_geom.dg_secbase)
 	{
+/* If the sector size has changed, trash all buffers because they're going
+ * to be the wrong size */
+		if (geom->dg_secsize != rcself->rc_geom.dg_secsize)
+		{
+			if (rcself->rc_sectorbuf)
+			{
+				dsk_free(rcself->rc_sectorbuf);
+				rcself->rc_sectorbuf
+					= dsk_malloc(geom->dg_secsize);
+				if (!rcself->rc_sectorbuf) return DSK_ERR_NOMEM;
+				memset(rcself->rc_sectorbuf, 0xE5,
+						geom->dg_secsize);
+			}
+			rcpmfs_free_buffers(rcself);
+		}
 		memcpy(&rcself->rc_geom, geom, sizeof(DSK_GEOMETRY));
-    /* Recreate .libdsk.ini 
-     * TODO: We can recreate the geometry; but what of the CP/M filesystem
-     * parameters? */
+    /* Recreate .libdsk.ini */
 		filename = rcpmfs_mkname(rcself, CONFIGFILE);
 		fp = fopen(filename, "w");
 		if (fp)
@@ -2051,4 +2133,161 @@ dsk_err_t rcpmfs_secid(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
 	return DSK_ERR_OK;
 }
 
+
+static dsk_err_t rcpmfs_update_config(RCPMFS_DSK_DRIVER *self)
+{
+	char *filename;
+	FILE *fp;
+	dsk_err_t err;
+
+	/* System option changed. Update the configuration file. */
+	filename = rcpmfs_mkname(self, CONFIGFILE);
+	fp = fopen(filename, "w");
+	if (!fp) return DSK_ERR_SYSERR;
+	err = rcpmfs_dump_options(self, fp);
+	fclose(fp);
+	if (err) return err;
+
+
+	return rcpmfs_readdir(self);
+}
+
+/* CP/M-specific filesystem parameters */
+static char *option_names[] = 
+{
+	"FS:CP/M:BSH", "FS:CP/M:BLM", "FS:CP/M:EXM",
+	"FS:CP/M:DSM", "FS:CP/M:DRM", "FS:CP/M:AL0", "FS:CP/M:AL1",
+	"FS:CP/M:CKS", "FS:CP/M:OFF", "FS:CP/M:VERSION"
+};
+
+#define MAXOPTION (sizeof(option_names) / sizeof(option_names[0]))
+
+dsk_err_t rcpmfs_option_enum(DSK_DRIVER *self, int idx, char **optname)
+{
+	if (!self) return DSK_ERR_BADPTR;
+	if (self->dr_class != &dc_rcpmfs) return DSK_ERR_BADPTR;
+
+	if (idx >= 0 && idx < MAXOPTION)
+	{
+		if (optname) *optname = option_names[idx];
+		return DSK_ERR_OK;
+	}
+	return DSK_ERR_BADOPT;	
+
+}
+
+/* Set a driver-specific option */
+dsk_err_t rcpmfs_option_set(DSK_DRIVER *self, const char *optname, int value)
+{
+	RCPMFS_DSK_DRIVER *rcpmfs_self;
+	int idx;
+	unsigned dirents;
+
+	if (!self || !optname) return DSK_ERR_BADPTR;
+	if (self->dr_class != &dc_rcpmfs) return DSK_ERR_BADPTR;
+	rcpmfs_self = (RCPMFS_DSK_DRIVER *)self;
+
+	for (idx = 0; idx < MAXOPTION; idx++)
+	{
+		if (!strcmp(optname, option_names[idx]))
+			break;
+	}
+	if (idx >= MAXOPTION) return DSK_ERR_BADOPT;
+
+	dirents = (rcpmfs_self->rc_dirblocks * rcpmfs_self->rc_blocksize) / 32;
+	switch(idx)
+	{
+
+		/* If we change the block size, try to keep the directory
+		 * size constant.
+		 *
+		 * If setting an option doesn't actually change anything, 
+		 * return and don't bother rewriting .libdsk.ini 
+		 */
+		case 0: if (rcpmfs_self->rc_blocksize == (128 << value))
+				return DSK_ERR_OK;
+			rcpmfs_self->rc_blocksize = (128 << value);
+			rcpmfs_self->rc_dirblocks = (dirents * 32) / 
+				rcpmfs_self->rc_blocksize;
+			break;
+		case 1: if (rcpmfs_self->rc_blocksize == (128 * (1+value)))
+				return DSK_ERR_OK;
+			rcpmfs_self->rc_blocksize = 128 * (1 + value);
+			rcpmfs_self->rc_dirblocks = (dirents * 32) / 
+				rcpmfs_self->rc_blocksize;
+			break;
+		case 2: return DSK_ERR_RDONLY;	/* EXM can't be changed */
+		case 3: if (rcpmfs_self->rc_totalblocks == (1 + value))
+				return DSK_ERR_OK;
+			rcpmfs_self->rc_totalblocks = value + 1;
+			break;
+		case 4: dirents = (32 * (value+1)) / rcpmfs_self->rc_blocksize;
+			if (rcpmfs_self->rc_dirblocks == dirents)
+				return DSK_ERR_OK;
+			rcpmfs_self->rc_dirblocks = dirents;
+			break;
+		case 5: return DSK_ERR_RDONLY; /* AL0 can't be changed. */
+		case 6: return DSK_ERR_RDONLY; /* AL1 can't be changed. */
+		case 7: return DSK_ERR_RDONLY; /* CKS can't be changed. */
+		case 8: if (rcpmfs_self->rc_systracks == value) 
+				return DSK_ERR_OK;
+			rcpmfs_self->rc_systracks = value;
+			break;
+		case 9:	if (rcpmfs_self->rc_fsversion == value)
+				return DSK_ERR_OK;
+			rcpmfs_self->rc_fsversion = value;
+			break;
+	}
+	return rcpmfs_update_config(rcpmfs_self);
+}
+
+
+dsk_err_t rcpmfs_option_get(DSK_DRIVER *self, const char *optname, int *value)
+{
+	RCPMFS_DSK_DRIVER *rcpmfs_self;
+	int idx, v = 0;
+	unsigned al, dirents;
+
+	if (!self || !optname) return DSK_ERR_BADPTR;
+	if (self->dr_class != &dc_rcpmfs) return DSK_ERR_BADPTR;
+	rcpmfs_self = (RCPMFS_DSK_DRIVER *)self;
+
+	for (idx = 0; idx < MAXOPTION; idx++)
+	{
+		if (!strcmp(optname, option_names[idx]))
+			break;
+	}
+	if (idx >= MAXOPTION) return DSK_ERR_BADOPT;
+
+	al = (1 << 16) - (1 << (16 - rcpmfs_self->rc_dirblocks));
+	dirents = (rcpmfs_self->rc_dirblocks * rcpmfs_self->rc_blocksize) / 32;
+	switch(idx)
+	{
+		case 0: v = dsk_get_psh(rcpmfs_self->rc_blocksize); // BSH
+			break;
+		case 1: v = (rcpmfs_self->rc_blocksize / 128) - 1; // BLM
+			break;
+		case 2: v = rcpmfs_get_exm(rcpmfs_self);	// EXM
+			break;
+		case 3: v = rcpmfs_self->rc_totalblocks - 1;	// DSM
+			break;
+		case 4: v = dirents - 1;
+			break;
+		case 5: v = (al >> 8) & 0xFF;	// AL0
+			break;
+		case 6: v = (al & 0xFF);	// AL1
+			break;
+		case 7: v = (dirents / 4);	// CKS
+			break;
+		case 8: v = rcpmfs_self->rc_systracks;	// OFF
+			break;
+		case 9: v = rcpmfs_self->rc_fsversion;	// Filesystem version
+			break;
+	}
+	if (value) *value = v;
+	return DSK_ERR_OK;
+}
+
+
+
 #endif /* def HAVE_RCPMFS */
